/* A Bison parser, made by GNU Bison 3.8.2.  */

/* Bison interface for Yacc-like parsers in C

   Copyright (C) 1984, 1989-1990, 2000-2015, 2018-2021 Free Software Foundation,
   Inc.

   This program is free software: you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation, either version 3 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program.  If not, see <https://www.gnu.org/licenses/>.  */

/* As a special exception, you may create a larger work that contains
   part or all of the Bison parser skeleton and distribute that work
   under terms of your choice, so long as that work isn't itself a
   parser generator using the skeleton or a modified version thereof
   as a parser skeleton.  Alternatively, if you modify or redistribute
   the parser skeleton itself, you may (at your option) remove this
   special exception, which will cause the skeleton and the resulting
   Bison output files to be licensed under the GNU General Public
   License without this special exception.

   This special exception was added by the Free Software Foundation in
   version 2.2 of Bison.  */

/* DO NOT RELY ON FEATURES THAT ARE NOT DOCUMENTED in the manual,
   especially those whose name start with YY_ or yy_.  They are
   private implementation details that can be changed or removed.  */

#ifndef YY_ITEX2MML_YY_SRC_LIBLASEM_0_6_SO_0_6_0_P_ITEX2MML_TAB_H_INCLUDED
# define YY_ITEX2MML_YY_SRC_LIBLASEM_0_6_SO_0_6_0_P_ITEX2MML_TAB_H_INCLUDED
/* Debug traces.  */
#ifndef YYDEBUG
# define YYDEBUG 0
#endif
#if YYDEBUG
extern int itex2MML_yydebug;
#endif

/* Token kinds.  */
#ifndef YYTOKENTYPE
# define YYTOKENTYPE
  enum yytokentype
  {
    YYEMPTY = -2,
    YYEOF = 0,                     /* "end of file"  */
    YYerror = 256,                 /* error  */
    YYUNDEF = 257,                 /* "invalid token"  */
    TEXOVER = 258,                 /* TEXOVER  */
    TEXATOP = 259,                 /* TEXATOP  */
    CHAR = 260,                    /* CHAR  */
    STARTMATH = 261,               /* STARTMATH  */
    STARTDMATH = 262,              /* STARTDMATH  */
    ENDMATH = 263,                 /* ENDMATH  */
    MI = 264,                      /* MI  */
    MIB = 265,                     /* MIB  */
    MN = 266,                      /* MN  */
    MO = 267,                      /* MO  */
    SUP = 268,                     /* SUP  */
    SUB = 269,                     /* SUB  */
    MROWOPEN = 270,                /* MROWOPEN  */
    MROWCLOSE = 271,               /* MROWCLOSE  */
    LEFT = 272,                    /* LEFT  */
    RIGHT = 273,                   /* RIGHT  */
    BIG = 274,                     /* BIG  */
    BBIG = 275,                    /* BBIG  */
    BIGG = 276,                    /* BIGG  */
    BBIGG = 277,                   /* BBIGG  */
    BIGL = 278,                    /* BIGL  */
    BBIGL = 279,                   /* BBIGL  */
    BIGGL = 280,                   /* BIGGL  */
    BBIGGL = 281,                  /* BBIGGL  */
    FRAC = 282,                    /* FRAC  */
    TFRAC = 283,                   /* TFRAC  */
    OPERATORNAME = 284,            /* OPERATORNAME  */
    MATHOP = 285,                  /* MATHOP  */
    MATHBIN = 286,                 /* MATHBIN  */
    MATHREL = 287,                 /* MATHREL  */
    MOP = 288,                     /* MOP  */
    MOL = 289,                     /* MOL  */
    MOLL = 290,                    /* MOLL  */
    MOF = 291,                     /* MOF  */
    MOR = 292,                     /* MOR  */
    PERIODDELIM = 293,             /* PERIODDELIM  */
    OTHERDELIM = 294,              /* OTHERDELIM  */
    LEFTDELIM = 295,               /* LEFTDELIM  */
    RIGHTDELIM = 296,              /* RIGHTDELIM  */
    MOS = 297,                     /* MOS  */
    MOB = 298,                     /* MOB  */
    SQRT = 299,                    /* SQRT  */
    ROOT = 300,                    /* ROOT  */
    BINOM = 301,                   /* BINOM  */
    TBINOM = 302,                  /* TBINOM  */
    UNDER = 303,                   /* UNDER  */
    OVER = 304,                    /* OVER  */
    OVERBRACE = 305,               /* OVERBRACE  */
    UNDERLINE = 306,               /* UNDERLINE  */
    UNDERBRACE = 307,              /* UNDERBRACE  */
    UNDEROVER = 308,               /* UNDEROVER  */
    TENSOR = 309,                  /* TENSOR  */
    MULTI = 310,                   /* MULTI  */
    ARRAYALIGN = 311,              /* ARRAYALIGN  */
    COLUMNALIGN = 312,             /* COLUMNALIGN  */
    ARRAY = 313,                   /* ARRAY  */
    COLSEP = 314,                  /* COLSEP  */
    ROWSEP = 315,                  /* ROWSEP  */
    ARRAYOPTS = 316,               /* ARRAYOPTS  */
    COLLAYOUT = 317,               /* COLLAYOUT  */
    COLALIGN = 318,                /* COLALIGN  */
    ROWALIGN = 319,                /* ROWALIGN  */
    ALIGN = 320,                   /* ALIGN  */
    EQROWS = 321,                  /* EQROWS  */
    EQCOLS = 322,                  /* EQCOLS  */
    ROWLINES = 323,                /* ROWLINES  */
    COLLINES = 324,                /* COLLINES  */
    FRAME = 325,                   /* FRAME  */
    PADDING = 326,                 /* PADDING  */
    ATTRLIST = 327,                /* ATTRLIST  */
    ITALICS = 328,                 /* ITALICS  */
    SANS = 329,                    /* SANS  */
    TT = 330,                      /* TT  */
    BOLD = 331,                    /* BOLD  */
    BOXED = 332,                   /* BOXED  */
    SLASHED = 333,                 /* SLASHED  */
    RM = 334,                      /* RM  */
    BB = 335,                      /* BB  */
    ST = 336,                      /* ST  */
    END = 337,                     /* END  */
    BBLOWERCHAR = 338,             /* BBLOWERCHAR  */
    BBUPPERCHAR = 339,             /* BBUPPERCHAR  */
    BBDIGIT = 340,                 /* BBDIGIT  */
    CALCHAR = 341,                 /* CALCHAR  */
    FRAKCHAR = 342,                /* FRAKCHAR  */
    CAL = 343,                     /* CAL  */
    SCR = 344,                     /* SCR  */
    FRAK = 345,                    /* FRAK  */
    CLAP = 346,                    /* CLAP  */
    LLAP = 347,                    /* LLAP  */
    RLAP = 348,                    /* RLAP  */
    ROWOPTS = 349,                 /* ROWOPTS  */
    TEXTSIZE = 350,                /* TEXTSIZE  */
    SCSIZE = 351,                  /* SCSIZE  */
    SCSCSIZE = 352,                /* SCSCSIZE  */
    DISPLAY = 353,                 /* DISPLAY  */
    TEXTSTY = 354,                 /* TEXTSTY  */
    TEXTBOX = 355,                 /* TEXTBOX  */
    TEXTSTRING = 356,              /* TEXTSTRING  */
    XMLSTRING = 357,               /* XMLSTRING  */
    CELLOPTS = 358,                /* CELLOPTS  */
    ROWSPAN = 359,                 /* ROWSPAN  */
    COLSPAN = 360,                 /* COLSPAN  */
    THINSPACE = 361,               /* THINSPACE  */
    MEDSPACE = 362,                /* MEDSPACE  */
    THICKSPACE = 363,              /* THICKSPACE  */
    QUAD = 364,                    /* QUAD  */
    QQUAD = 365,                   /* QQUAD  */
    NEGSPACE = 366,                /* NEGSPACE  */
    NEGMEDSPACE = 367,             /* NEGMEDSPACE  */
    NEGTHICKSPACE = 368,           /* NEGTHICKSPACE  */
    PHANTOM = 369,                 /* PHANTOM  */
    HREF = 370,                    /* HREF  */
    UNKNOWNCHAR = 371,             /* UNKNOWNCHAR  */
    EMPTYMROW = 372,               /* EMPTYMROW  */
    STATLINE = 373,                /* STATLINE  */
    TOOLTIP = 374,                 /* TOOLTIP  */
    TOGGLE = 375,                  /* TOGGLE  */
    TOGGLESTART = 376,             /* TOGGLESTART  */
    TOGGLEEND = 377,               /* TOGGLEEND  */
    FGHIGHLIGHT = 378,             /* FGHIGHLIGHT  */
    BGHIGHLIGHT = 379,             /* BGHIGHLIGHT  */
    SPACE = 380,                   /* SPACE  */
    INTONE = 381,                  /* INTONE  */
    INTTWO = 382,                  /* INTTWO  */
    INTTHREE = 383,                /* INTTHREE  */
    BAR = 384,                     /* BAR  */
    WIDEBAR = 385,                 /* WIDEBAR  */
    VEC = 386,                     /* VEC  */
    WIDEVEC = 387,                 /* WIDEVEC  */
    WIDELVEC = 388,                /* WIDELVEC  */
    WIDELRVEC = 389,               /* WIDELRVEC  */
    WIDEUVEC = 390,                /* WIDEUVEC  */
    WIDEULVEC = 391,               /* WIDEULVEC  */
    WIDEULRVEC = 392,              /* WIDEULRVEC  */
    HAT = 393,                     /* HAT  */
    WIDEHAT = 394,                 /* WIDEHAT  */
    CHECK = 395,                   /* CHECK  */
    WIDECHECK = 396,               /* WIDECHECK  */
    TILDE = 397,                   /* TILDE  */
    WIDETILDE = 398,               /* WIDETILDE  */
    DOT = 399,                     /* DOT  */
    DDOT = 400,                    /* DDOT  */
    DDDOT = 401,                   /* DDDOT  */
    DDDDOT = 402,                  /* DDDDOT  */
    UNARYMINUS = 403,              /* UNARYMINUS  */
    UNARYPLUS = 404,               /* UNARYPLUS  */
    BEGINENV = 405,                /* BEGINENV  */
    ENDENV = 406,                  /* ENDENV  */
    MATRIX = 407,                  /* MATRIX  */
    PMATRIX = 408,                 /* PMATRIX  */
    BMATRIX = 409,                 /* BMATRIX  */
    BBMATRIX = 410,                /* BBMATRIX  */
    VMATRIX = 411,                 /* VMATRIX  */
    VVMATRIX = 412,                /* VVMATRIX  */
    SVG = 413,                     /* SVG  */
    ENDSVG = 414,                  /* ENDSVG  */
    SMALLMATRIX = 415,             /* SMALLMATRIX  */
    CASES = 416,                   /* CASES  */
    ALIGNED = 417,                 /* ALIGNED  */
    GATHERED = 418,                /* GATHERED  */
    SUBSTACK = 419,                /* SUBSTACK  */
    PMOD = 420,                    /* PMOD  */
    RMCHAR = 421,                  /* RMCHAR  */
    COLOR = 422,                   /* COLOR  */
    BGCOLOR = 423,                 /* BGCOLOR  */
    XARROW = 424,                  /* XARROW  */
    OPTARGOPEN = 425,              /* OPTARGOPEN  */
    OPTARGCLOSE = 426,             /* OPTARGCLOSE  */
    ITEXNUM = 427,                 /* ITEXNUM  */
    RAISEBOX = 428,                /* RAISEBOX  */
    NEG = 429                      /* NEG  */
  };
  typedef enum yytokentype yytoken_kind_t;
#endif
/* Token kinds.  */
#define YYEMPTY -2
#define YYEOF 0
#define YYerror 256
#define YYUNDEF 257
#define TEXOVER 258
#define TEXATOP 259
#define CHAR 260
#define STARTMATH 261
#define STARTDMATH 262
#define ENDMATH 263
#define MI 264
#define MIB 265
#define MN 266
#define MO 267
#define SUP 268
#define SUB 269
#define MROWOPEN 270
#define MROWCLOSE 271
#define LEFT 272
#define RIGHT 273
#define BIG 274
#define BBIG 275
#define BIGG 276
#define BBIGG 277
#define BIGL 278
#define BBIGL 279
#define BIGGL 280
#define BBIGGL 281
#define FRAC 282
#define TFRAC 283
#define OPERATORNAME 284
#define MATHOP 285
#define MATHBIN 286
#define MATHREL 287
#define MOP 288
#define MOL 289
#define MOLL 290
#define MOF 291
#define MOR 292
#define PERIODDELIM 293
#define OTHERDELIM 294
#define LEFTDELIM 295
#define RIGHTDELIM 296
#define MOS 297
#define MOB 298
#define SQRT 299
#define ROOT 300
#define BINOM 301
#define TBINOM 302
#define UNDER 303
#define OVER 304
#define OVERBRACE 305
#define UNDERLINE 306
#define UNDERBRACE 307
#define UNDEROVER 308
#define TENSOR 309
#define MULTI 310
#define ARRAYALIGN 311
#define COLUMNALIGN 312
#define ARRAY 313
#define COLSEP 314
#define ROWSEP 315
#define ARRAYOPTS 316
#define COLLAYOUT 317
#define COLALIGN 318
#define ROWALIGN 319
#define ALIGN 320
#define EQROWS 321
#define EQCOLS 322
#define ROWLINES 323
#define COLLINES 324
#define FRAME 325
#define PADDING 326
#define ATTRLIST 327
#define ITALICS 328
#define SANS 329
#define TT 330
#define BOLD 331
#define BOXED 332
#define SLASHED 333
#define RM 334
#define BB 335
#define ST 336
#define END 337
#define BBLOWERCHAR 338
#define BBUPPERCHAR 339
#define BBDIGIT 340
#define CALCHAR 341
#define FRAKCHAR 342
#define CAL 343
#define SCR 344
#define FRAK 345
#define CLAP 346
#define LLAP 347
#define RLAP 348
#define ROWOPTS 349
#define TEXTSIZE 350
#define SCSIZE 351
#define SCSCSIZE 352
#define DISPLAY 353
#define TEXTSTY 354
#define TEXTBOX 355
#define TEXTSTRING 356
#define XMLSTRING 357
#define CELLOPTS 358
#define ROWSPAN 359
#define COLSPAN 360
#define THINSPACE 361
#define MEDSPACE 362
#define THICKSPACE 363
#define QUAD 364
#define QQUAD 365
#define NEGSPACE 366
#define NEGMEDSPACE 367
#define NEGTHICKSPACE 368
#define PHANTOM 369
#define HREF 370
#define UNKNOWNCHAR 371
#define EMPTYMROW 372
#define STATLINE 373
#define TOOLTIP 374
#define TOGGLE 375
#define TOGGLESTART 376
#define TOGGLEEND 377
#define FGHIGHLIGHT 378
#define BGHIGHLIGHT 379
#define SPACE 380
#define INTONE 381
#define INTTWO 382
#define INTTHREE 383
#define BAR 384
#define WIDEBAR 385
#define VEC 386
#define WIDEVEC 387
#define WIDELVEC 388
#define WIDELRVEC 389
#define WIDEUVEC 390
#define WIDEULVEC 391
#define WIDEULRVEC 392
#define HAT 393
#define WIDEHAT 394
#define CHECK 395
#define WIDECHECK 396
#define TILDE 397
#define WIDETILDE 398
#define DOT 399
#define DDOT 400
#define DDDOT 401
#define DDDDOT 402
#define UNARYMINUS 403
#define UNARYPLUS 404
#define BEGINENV 405
#define ENDENV 406
#define MATRIX 407
#define PMATRIX 408
#define BMATRIX 409
#define BBMATRIX 410
#define VMATRIX 411
#define VVMATRIX 412
#define SVG 413
#define ENDSVG 414
#define SMALLMATRIX 415
#define CASES 416
#define ALIGNED 417
#define GATHERED 418
#define SUBSTACK 419
#define PMOD 420
#define RMCHAR 421
#define COLOR 422
#define BGCOLOR 423
#define XARROW 424
#define OPTARGOPEN 425
#define OPTARGCLOSE 426
#define ITEXNUM 427
#define RAISEBOX 428
#define NEG 429

/* Value type.  */
#if ! defined YYSTYPE && ! defined YYSTYPE_IS_DECLARED
typedef int YYSTYPE;
# define YYSTYPE_IS_TRIVIAL 1
# define YYSTYPE_IS_DECLARED 1
#endif


extern YYSTYPE itex2MML_yylval;


int itex2MML_yyparse (char **ret_str);


#endif /* !YY_ITEX2MML_YY_SRC_LIBLASEM_0_6_SO_0_6_0_P_ITEX2MML_TAB_H_INCLUDED  */
