/* Lasem - SVG and Mathml library
 *
 * Copyright © 2007-2008 Emmanuel Pacaud
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Lesser General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General
 * Public License along with this library; if not, write to the
 * Free Software Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA 02110-1335, USA.
 *
 * Author:
 * 	Emmanuel Pacaud <emmanuel@gnome.org>
 */

#ifndef LSM_MATHML_TYPES_H
#define LSM_MATHML_TYPES_H

#include <lsmmathmlenums.h>

G_BEGIN_DECLS

typedef struct _LsmMathmlDocument LsmMathmlDocument;
typedef struct _LsmMathmlElement LsmMathmlElement;
typedef struct _LsmMathmlSemanticsElement LsmMathmlSemanticsElement;
typedef struct _LsmMathmlPhantomElement LsmMathmlPhantomElement;
typedef struct _LsmMathmlTableElement LsmMathmlTableElement;
typedef struct _LsmMathmlTableRowElement LsmMathmlTableRowElement;
typedef struct _LsmMathmlTableCellElement LsmMathmlTableCellElement;
typedef struct _LsmMathmlSpaceElement LsmMathmlSpaceElement;
typedef struct _LsmMathmlFractionElement LsmMathmlFractionElement;
typedef struct _LsmMathmlUnderOverElement LsmMathmlUnderOverElement;
typedef struct _LsmMathmlMathElement LsmMathmlMathElement;
typedef struct _LsmMathmlRadicalElement LsmMathmlRadicalElement;
typedef struct _LsmMathmlScriptElement LsmMathmlScriptElement;
typedef struct _LsmMathmlPresentationToken LsmMathmlPresentationToken;
typedef struct _LsmMathmlNumberElement LsmMathmlNumberElement;
typedef struct _LsmMathmlOperatorElement LsmMathmlOperatorElement;
typedef struct _LsmMathmlStringElement LsmMathmlStringElement;
typedef struct _LsmMathmlPresentationContainer LsmMathmlPresentationContainer;
typedef struct _LsmMathmlStyleElement LsmMathmlStyleElement;
typedef struct _LsmMathmlRowElement LsmMathmlRowElement;
typedef struct _LsmMathmlEncloseElement LsmMathmlEncloseElement;
typedef struct _LsmMathmlFencedElement LsmMathmlFencedElement;
typedef struct _LsmMathmlPaddedElement LsmMathmlPaddedElement;
typedef struct _LsmMathmlErrorElement LsmMathmlErrorElement;
typedef struct _LsmMathmlActionElement LsmMathmlActionElement;
typedef struct _LsmMathmlAlignMarkElement LsmMathmlAlignMarkElement;
typedef struct _LsmMathmlAlignGroupElement LsmMathmlAlignGroupElement;

typedef struct _LsmMathmlItexElement LsmMathmlItexElement;

typedef struct _LsmMathmlView LsmMathmlView;

typedef struct _LsmMathmlStyle LsmMathmlStyle;

G_END_DECLS

#endif
