
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */

#include "lsmmathmlenumtypes.h"
#include "lsmmathml.h"
#include "lsmmathmltypes.h"
#include "lsmmathmlenums.h"
#include "lsmmathmltraits.h"
#include "lsmmathmlattributes.h"
#include "lsmmathmloperatordictionary.h"
#include "lsmmathmldocument.h"
#include "lsmmathmlelement.h"
#include "lsmmathmlsemanticselement.h"
#include "lsmmathmlphantomelement.h"
#include "lsmmathmlmathelement.h"
#include "lsmmathmltableelement.h"
#include "lsmmathmltablerowelement.h"
#include "lsmmathmltablecellelement.h"
#include "lsmmathmlspaceelement.h"
#include "lsmmathmlradicalelement.h"
#include "lsmmathmlscriptelement.h"
#include "lsmmathmlfractionelement.h"
#include "lsmmathmlunderoverelement.h"
#include "lsmmathmlpresentationtoken.h"
#include "lsmmathmloperatorelement.h"
#include "lsmmathmlstringelement.h"
#include "lsmmathmlpresentationcontainer.h"
#include "lsmmathmlstyleelement.h"
#include "lsmmathmlrowelement.h"
#include "lsmmathmlencloseelement.h"
#include "lsmmathmlfencedelement.h"
#include "lsmmathmlpaddedelement.h"
#include "lsmmathmlerrorelement.h"
#include "lsmmathmlactionelement.h"
#include "lsmmathmlstyle.h"
#include "lsmmathmlview.h"
#include "lsmmathmlglyphtableams.h"
#include "lsmmathmlalignmarkelement.h"
#include "lsmmathmlaligngroupelement.h"
#include "lsmmathmlitexelement.h"
#include "lsmmathmlutils.h"
#include "lsmmathmllayoututils.h"

#define C_ENUM(v) ((gint) v)
#define C_FLAGS(v) ((guint) v)

/* enumerations from "lsmmathmlenums.h" */

GType
lsm_mathml_css_type_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (LSM_MATHML_CSS_TYPE_USER), "LSM_MATHML_CSS_TYPE_USER", "user" },
        { C_ENUM (LSM_MATHML_CSS_TYPE_AUTHOR), "LSM_MATHML_CSS_TYPE_AUTHOR", "author" },
        { C_ENUM (LSM_MATHML_CSS_TYPE_AUTHOR_IMPORTANT), "LSM_MATHML_CSS_TYPE_AUTHOR_IMPORTANT", "author-important" },
        { C_ENUM (LSM_MATHML_CSS_TYPE_USER_IMPORTANT), "LSM_MATHML_CSS_TYPE_USER_IMPORTANT", "user-important" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("LsmMathmlCssType"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
lsm_mathml_mode_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (LSM_MATHML_MODE_ERROR), "LSM_MATHML_MODE_ERROR", "error" },
        { C_ENUM (LSM_MATHML_MODE_DISPLAY), "LSM_MATHML_MODE_DISPLAY", "display" },
        { C_ENUM (LSM_MATHML_MODE_INLINE), "LSM_MATHML_MODE_INLINE", "inline" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("LsmMathmlMode"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
lsm_mathml_display_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (LSM_MATHML_DISPLAY_ERROR), "LSM_MATHML_DISPLAY_ERROR", "error" },
        { C_ENUM (LSM_MATHML_DISPLAY_BLOCK), "LSM_MATHML_DISPLAY_BLOCK", "block" },
        { C_ENUM (LSM_MATHML_DISPLAY_INLINE), "LSM_MATHML_DISPLAY_INLINE", "inline" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("LsmMathmlDisplay"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
lsm_mathml_space_name_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (LSM_MATHML_SPACE_NAME_ERROR), "LSM_MATHML_SPACE_NAME_ERROR", "error" },
        { C_ENUM (LSM_MATHML_SPACE_NAME_NEGATIVE_VERY_VERY_THIN), "LSM_MATHML_SPACE_NAME_NEGATIVE_VERY_VERY_THIN", "negative-very-very-thin" },
        { C_ENUM (LSM_MATHML_SPACE_NAME_NEGATIVE_VERY_THIN), "LSM_MATHML_SPACE_NAME_NEGATIVE_VERY_THIN", "negative-very-thin" },
        { C_ENUM (LSM_MATHML_SPACE_NAME_NEGATIVE_THIN), "LSM_MATHML_SPACE_NAME_NEGATIVE_THIN", "negative-thin" },
        { C_ENUM (LSM_MATHML_SPACE_NAME_NEGATIVE_MEDIUM), "LSM_MATHML_SPACE_NAME_NEGATIVE_MEDIUM", "negative-medium" },
        { C_ENUM (LSM_MATHML_SPACE_NAME_NEGATIVE_THICK), "LSM_MATHML_SPACE_NAME_NEGATIVE_THICK", "negative-thick" },
        { C_ENUM (LSM_MATHML_SPACE_NAME_NEGATIVE_VERY_THICK), "LSM_MATHML_SPACE_NAME_NEGATIVE_VERY_THICK", "negative-very-thick" },
        { C_ENUM (LSM_MATHML_SPACE_NAME_NEGATIVE_VERY_VERY_THICK), "LSM_MATHML_SPACE_NAME_NEGATIVE_VERY_VERY_THICK", "negative-very-very-thick" },
        { C_ENUM (LSM_MATHML_SPACE_NAME_VERY_VERY_THIN), "LSM_MATHML_SPACE_NAME_VERY_VERY_THIN", "very-very-thin" },
        { C_ENUM (LSM_MATHML_SPACE_NAME_VERY_THIN), "LSM_MATHML_SPACE_NAME_VERY_THIN", "very-thin" },
        { C_ENUM (LSM_MATHML_SPACE_NAME_THIN), "LSM_MATHML_SPACE_NAME_THIN", "thin" },
        { C_ENUM (LSM_MATHML_SPACE_NAME_MEDIUM), "LSM_MATHML_SPACE_NAME_MEDIUM", "medium" },
        { C_ENUM (LSM_MATHML_SPACE_NAME_THICK), "LSM_MATHML_SPACE_NAME_THICK", "thick" },
        { C_ENUM (LSM_MATHML_SPACE_NAME_VERY_THICK), "LSM_MATHML_SPACE_NAME_VERY_THICK", "very-thick" },
        { C_ENUM (LSM_MATHML_SPACE_NAME_VERY_VERY_THICK), "LSM_MATHML_SPACE_NAME_VERY_VERY_THICK", "very-very-thick" },
        { C_ENUM (LSM_MATHML_SPACE_NAME_INFINITY), "LSM_MATHML_SPACE_NAME_INFINITY", "infinity" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("LsmMathmlSpaceName"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
lsm_mathml_unit_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (LSM_MATHML_UNIT_ERROR), "LSM_MATHML_UNIT_ERROR", "error" },
        { C_ENUM (LSM_MATHML_UNIT_NONE), "LSM_MATHML_UNIT_NONE", "none" },
        { C_ENUM (LSM_MATHML_UNIT_EM), "LSM_MATHML_UNIT_EM", "em" },
        { C_ENUM (LSM_MATHML_UNIT_EX), "LSM_MATHML_UNIT_EX", "ex" },
        { C_ENUM (LSM_MATHML_UNIT_IN), "LSM_MATHML_UNIT_IN", "in" },
        { C_ENUM (LSM_MATHML_UNIT_CM), "LSM_MATHML_UNIT_CM", "cm" },
        { C_ENUM (LSM_MATHML_UNIT_MM), "LSM_MATHML_UNIT_MM", "mm" },
        { C_ENUM (LSM_MATHML_UNIT_PT), "LSM_MATHML_UNIT_PT", "pt" },
        { C_ENUM (LSM_MATHML_UNIT_PX), "LSM_MATHML_UNIT_PX", "px" },
        { C_ENUM (LSM_MATHML_UNIT_PC), "LSM_MATHML_UNIT_PC", "pc" },
        { C_ENUM (LSM_MATHML_UNIT_PERCENT), "LSM_MATHML_UNIT_PERCENT", "percent" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("LsmMathmlUnit"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
lsm_mathml_font_style_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (LSM_MATHML_FONT_STYLE_ERROR), "LSM_MATHML_FONT_STYLE_ERROR", "error" },
        { C_ENUM (LSM_MATHML_FONT_STYLE_NORMAL), "LSM_MATHML_FONT_STYLE_NORMAL", "normal" },
        { C_ENUM (LSM_MATHML_FONT_STYLE_ITALIC), "LSM_MATHML_FONT_STYLE_ITALIC", "italic" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("LsmMathmlFontStyle"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
lsm_mathml_font_weight_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (LSM_MATHML_FONT_WEIGHT_ERROR), "LSM_MATHML_FONT_WEIGHT_ERROR", "error" },
        { C_ENUM (LSM_MATHML_FONT_WEIGHT_NORMAL), "LSM_MATHML_FONT_WEIGHT_NORMAL", "normal" },
        { C_ENUM (LSM_MATHML_FONT_WEIGHT_BOLD), "LSM_MATHML_FONT_WEIGHT_BOLD", "bold" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("LsmMathmlFontWeight"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
lsm_mathml_variant_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (LSM_MATHML_VARIANT_ERROR), "LSM_MATHML_VARIANT_ERROR", "error" },
        { C_ENUM (LSM_MATHML_VARIANT_NORMAL), "LSM_MATHML_VARIANT_NORMAL", "normal" },
        { C_ENUM (LSM_MATHML_VARIANT_BOLD), "LSM_MATHML_VARIANT_BOLD", "bold" },
        { C_ENUM (LSM_MATHML_VARIANT_ITALIC), "LSM_MATHML_VARIANT_ITALIC", "italic" },
        { C_ENUM (LSM_MATHML_VARIANT_BOLD_ITALIC), "LSM_MATHML_VARIANT_BOLD_ITALIC", "bold-italic" },
        { C_ENUM (LSM_MATHML_VARIANT_DOUBLE_STRUCK), "LSM_MATHML_VARIANT_DOUBLE_STRUCK", "double-struck" },
        { C_ENUM (LSM_MATHML_VARIANT_FRAKTUR_BOLD), "LSM_MATHML_VARIANT_FRAKTUR_BOLD", "fraktur-bold" },
        { C_ENUM (LSM_MATHML_VARIANT_SCRIPT), "LSM_MATHML_VARIANT_SCRIPT", "script" },
        { C_ENUM (LSM_MATHML_VARIANT_SCRIPT_BOLD), "LSM_MATHML_VARIANT_SCRIPT_BOLD", "script-bold" },
        { C_ENUM (LSM_MATHML_VARIANT_FRAKTUR), "LSM_MATHML_VARIANT_FRAKTUR", "fraktur" },
        { C_ENUM (LSM_MATHML_VARIANT_SANS_SERIF), "LSM_MATHML_VARIANT_SANS_SERIF", "sans-serif" },
        { C_ENUM (LSM_MATHML_VARIANT_SANS_SERIF_BOLD), "LSM_MATHML_VARIANT_SANS_SERIF_BOLD", "sans-serif-bold" },
        { C_ENUM (LSM_MATHML_VARIANT_SANS_SERIF_ITALIC), "LSM_MATHML_VARIANT_SANS_SERIF_ITALIC", "sans-serif-italic" },
        { C_ENUM (LSM_MATHML_VARIANT_SANS_SERIF_BOLD_ITALIC), "LSM_MATHML_VARIANT_SANS_SERIF_BOLD_ITALIC", "sans-serif-bold-italic" },
        { C_ENUM (LSM_MATHML_VARIANT_MONOSPACE), "LSM_MATHML_VARIANT_MONOSPACE", "monospace" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("LsmMathmlVariant"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
lsm_mathml_form_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (LSM_MATHML_FORM_ERROR), "LSM_MATHML_FORM_ERROR", "error" },
        { C_ENUM (LSM_MATHML_FORM_PREFIX), "LSM_MATHML_FORM_PREFIX", "prefix" },
        { C_ENUM (LSM_MATHML_FORM_POSTFIX), "LSM_MATHML_FORM_POSTFIX", "postfix" },
        { C_ENUM (LSM_MATHML_FORM_INFIX), "LSM_MATHML_FORM_INFIX", "infix" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("LsmMathmlForm"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
lsm_mathml_row_align_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (LSM_MATHML_ROW_ALIGN_ERROR), "LSM_MATHML_ROW_ALIGN_ERROR", "error" },
        { C_ENUM (LSM_MATHML_ROW_ALIGN_BASELINE), "LSM_MATHML_ROW_ALIGN_BASELINE", "baseline" },
        { C_ENUM (LSM_MATHML_ROW_ALIGN_TOP), "LSM_MATHML_ROW_ALIGN_TOP", "top" },
        { C_ENUM (LSM_MATHML_ROW_ALIGN_BOTTOM), "LSM_MATHML_ROW_ALIGN_BOTTOM", "bottom" },
        { C_ENUM (LSM_MATHML_ROW_ALIGN_CENTER), "LSM_MATHML_ROW_ALIGN_CENTER", "center" },
        { C_ENUM (LSM_MATHML_ROW_ALIGN_AXIS), "LSM_MATHML_ROW_ALIGN_AXIS", "axis" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("LsmMathmlRowAlign"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
lsm_mathml_column_align_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (LSM_MATHML_COLUMN_ALIGN_ERROR), "LSM_MATHML_COLUMN_ALIGN_ERROR", "error" },
        { C_ENUM (LSM_MATHML_COLUMN_ALIGN_CENTER), "LSM_MATHML_COLUMN_ALIGN_CENTER", "center" },
        { C_ENUM (LSM_MATHML_COLUMN_ALIGN_LEFT), "LSM_MATHML_COLUMN_ALIGN_LEFT", "left" },
        { C_ENUM (LSM_MATHML_COLUMN_ALIGN_RIGHT), "LSM_MATHML_COLUMN_ALIGN_RIGHT", "right" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("LsmMathmlColumnAlign"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
lsm_mathml_line_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (LSM_MATHML_LINE_ERROR), "LSM_MATHML_LINE_ERROR", "error" },
        { C_ENUM (LSM_MATHML_LINE_NONE), "LSM_MATHML_LINE_NONE", "none" },
        { C_ENUM (LSM_MATHML_LINE_SOLID), "LSM_MATHML_LINE_SOLID", "solid" },
        { C_ENUM (LSM_MATHML_LINE_DASHED), "LSM_MATHML_LINE_DASHED", "dashed" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("LsmMathmlLine"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
lsm_mathml_script_level_sign_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (LSM_MATHML_SCRIPT_LEVEL_SIGN_ERROR), "LSM_MATHML_SCRIPT_LEVEL_SIGN_ERROR", "error" },
        { C_ENUM (LSM_MATHML_SCRIPT_LEVEL_SIGN_NONE), "LSM_MATHML_SCRIPT_LEVEL_SIGN_NONE", "none" },
        { C_ENUM (LSM_MATHML_SCRIPT_LEVEL_SIGN_PLUS), "LSM_MATHML_SCRIPT_LEVEL_SIGN_PLUS", "plus" },
        { C_ENUM (LSM_MATHML_SCRIPT_LEVEL_SIGN_MINUS), "LSM_MATHML_SCRIPT_LEVEL_SIGN_MINUS", "minus" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("LsmMathmlScriptLevelSign"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
lsm_mathml_notation_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (LSM_MATHML_NOTATION_ERROR), "LSM_MATHML_NOTATION_ERROR", "error" },
        { C_ENUM (LSM_MATHML_NOTATION_LONGDIV), "LSM_MATHML_NOTATION_LONGDIV", "longdiv" },
        { C_ENUM (LSM_MATHML_NOTATION_ACTUARIAL), "LSM_MATHML_NOTATION_ACTUARIAL", "actuarial" },
        { C_ENUM (LSM_MATHML_NOTATION_RADICAL), "LSM_MATHML_NOTATION_RADICAL", "radical" },
        { C_ENUM (LSM_MATHML_NOTATION_BOX), "LSM_MATHML_NOTATION_BOX", "box" },
        { C_ENUM (LSM_MATHML_NOTATION_ROUNDED_BOX), "LSM_MATHML_NOTATION_ROUNDED_BOX", "rounded-box" },
        { C_ENUM (LSM_MATHML_NOTATION_CIRCLE), "LSM_MATHML_NOTATION_CIRCLE", "circle" },
        { C_ENUM (LSM_MATHML_NOTATION_LEFT), "LSM_MATHML_NOTATION_LEFT", "left" },
        { C_ENUM (LSM_MATHML_NOTATION_RIGHT), "LSM_MATHML_NOTATION_RIGHT", "right" },
        { C_ENUM (LSM_MATHML_NOTATION_TOP), "LSM_MATHML_NOTATION_TOP", "top" },
        { C_ENUM (LSM_MATHML_NOTATION_BOTTOM), "LSM_MATHML_NOTATION_BOTTOM", "bottom" },
        { C_ENUM (LSM_MATHML_NOTATION_UP_DIAGONAL_STRIKE), "LSM_MATHML_NOTATION_UP_DIAGONAL_STRIKE", "up-diagonal-strike" },
        { C_ENUM (LSM_MATHML_NOTATION_DOWN_DIAGONAL_STRIKE), "LSM_MATHML_NOTATION_DOWN_DIAGONAL_STRIKE", "down-diagonal-strike" },
        { C_ENUM (LSM_MATHML_NOTATION_VERTICAL_STRIKE), "LSM_MATHML_NOTATION_VERTICAL_STRIKE", "vertical-strike" },
        { C_ENUM (LSM_MATHML_NOTATION_HORIZONTAL_STRIKE), "LSM_MATHML_NOTATION_HORIZONTAL_STRIKE", "horizontal-strike" },
        { C_ENUM (LSM_MATHML_NOTATION_MADRUWB), "LSM_MATHML_NOTATION_MADRUWB", "madruwb" },
        { C_ENUM (LSM_MATHML_NOTATION_UP_DIAGONAL_ARROW), "LSM_MATHML_NOTATION_UP_DIAGONAL_ARROW", "up-diagonal-arrow" },
        { C_ENUM (LSM_MATHML_NOTATION_LAST), "LSM_MATHML_NOTATION_LAST", "last" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("LsmMathmlNotation"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
lsm_mathml_linebreak_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (LSM_MATHML_LINEBREAK_ERROR), "LSM_MATHML_LINEBREAK_ERROR", "error" },
        { C_ENUM (LSM_MATHML_LINEBREAK_AUTO), "LSM_MATHML_LINEBREAK_AUTO", "auto" },
        { C_ENUM (LSM_MATHML_LINEBREAK_NEWLINE), "LSM_MATHML_LINEBREAK_NEWLINE", "newline" },
        { C_ENUM (LSM_MATHML_LINEBREAK_NOBREAK), "LSM_MATHML_LINEBREAK_NOBREAK", "nobreak" },
        { C_ENUM (LSM_MATHML_LINEBREAK_GOODBREAK), "LSM_MATHML_LINEBREAK_GOODBREAK", "goodbreak" },
        { C_ENUM (LSM_MATHML_LINEBREAK_BADBREAK), "LSM_MATHML_LINEBREAK_BADBREAK", "badbreak" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("LsmMathmlLinebreak"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "lsmmathmlpresentationtoken.h" */

GType
lsm_mathml_presentation_token_type_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (LSM_MATHML_PRESENTATION_TOKEN_TYPE_NUMBER), "LSM_MATHML_PRESENTATION_TOKEN_TYPE_NUMBER", "number" },
        { C_ENUM (LSM_MATHML_PRESENTATION_TOKEN_TYPE_IDENTIFIER), "LSM_MATHML_PRESENTATION_TOKEN_TYPE_IDENTIFIER", "identifier" },
        { C_ENUM (LSM_MATHML_PRESENTATION_TOKEN_TYPE_TEXT), "LSM_MATHML_PRESENTATION_TOKEN_TYPE_TEXT", "text" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("LsmMathmlPresentationTokenType"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "lsmmathmlradicalelement.h" */

GType
lsm_mathml_radical_element_type_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (LSM_MATHML_RADICAL_ELEMENT_TYPE_SQRT), "LSM_MATHML_RADICAL_ELEMENT_TYPE_SQRT", "sqrt" },
        { C_ENUM (LSM_MATHML_RADICAL_ELEMENT_TYPE_ROOT), "LSM_MATHML_RADICAL_ELEMENT_TYPE_ROOT", "root" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("LsmMathmlRadicalElementType"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "lsmmathmlscriptelement.h" */

GType
lsm_mathml_script_element_type_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (LSM_MATHML_SCRIPT_ELEMENT_TYPE_SUB), "LSM_MATHML_SCRIPT_ELEMENT_TYPE_SUB", "sub" },
        { C_ENUM (LSM_MATHML_SCRIPT_ELEMENT_TYPE_SUP), "LSM_MATHML_SCRIPT_ELEMENT_TYPE_SUP", "sup" },
        { C_ENUM (LSM_MATHML_SCRIPT_ELEMENT_TYPE_SUB_SUP), "LSM_MATHML_SCRIPT_ELEMENT_TYPE_SUB_SUP", "sub-sup" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("LsmMathmlScriptElementType"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "lsmmathmltablerowelement.h" */

GType
lsm_mathml_table_row_element_type_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (LSM_MATHML_TABLE_ROW_ELEMENT_TYPE_ROW), "LSM_MATHML_TABLE_ROW_ELEMENT_TYPE_ROW", "row" },
        { C_ENUM (LSM_MATHML_TABLE_ROW_ELEMENT_TYPE_LABELED_ROW), "LSM_MATHML_TABLE_ROW_ELEMENT_TYPE_LABELED_ROW", "labeled-row" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("LsmMathmlTableRowElementType"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "lsmmathmlunderoverelement.h" */

GType
lsm_mathml_under_over_element_type_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (LSM_MATHML_UNDER_OVER_ELEMENT_TYPE_UNDER), "LSM_MATHML_UNDER_OVER_ELEMENT_TYPE_UNDER", "under" },
        { C_ENUM (LSM_MATHML_UNDER_OVER_ELEMENT_TYPE_OVER), "LSM_MATHML_UNDER_OVER_ELEMENT_TYPE_OVER", "over" },
        { C_ENUM (LSM_MATHML_UNDER_OVER_ELEMENT_TYPE_UNDER_OVER), "LSM_MATHML_UNDER_OVER_ELEMENT_TYPE_UNDER_OVER", "under-over" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("LsmMathmlUnderOverElementType"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "lsmmathmlview.h" */

GType
lsm_mathml_glyph_flags_get_type (void)
{
    static gsize gtype_id = 0;
    static const GFlagsValue values[] = {
        { C_FLAGS (LSM_MATHML_GLYPH_FLAG_STRETCH_VERTICAL), "LSM_MATHML_GLYPH_FLAG_STRETCH_VERTICAL", "stretch-vertical" },
        { C_FLAGS (LSM_MATHML_GLYPH_FLAG_STRETCH_HORIZONTAL), "LSM_MATHML_GLYPH_FLAG_STRETCH_HORIZONTAL", "stretch-horizontal" },
        { C_FLAGS (LSM_MATHML_GLYPH_FLAG_TYPE_SIZED), "LSM_MATHML_GLYPH_FLAG_TYPE_SIZED", "type-sized" },
        { C_FLAGS (LSM_MATHML_GLYPH_FLAG_ALIGN_AXIS), "LSM_MATHML_GLYPH_FLAG_ALIGN_AXIS", "align-axis" },
        { C_FLAGS (LSM_MATHML_GLYPH_FLAG_HAS_LARGE_VERSION), "LSM_MATHML_GLYPH_FLAG_HAS_LARGE_VERSION", "has-large-version" },
        { C_FLAGS (LSM_MATHML_GLYPH_FLAG_INTEGRAL_SLANT), "LSM_MATHML_GLYPH_FLAG_INTEGRAL_SLANT", "integral-slant" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_flags_register_static (g_intern_static_string ("LsmMathmlGlyphFlags"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

GType
lsm_mathml_font_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (LSM_MATHML_FONT_ERROR), "LSM_MATHML_FONT_ERROR", "error" },
        { C_ENUM (LSM_MATHML_FONT_DEFAULT), "LSM_MATHML_FONT_DEFAULT", "default" },
        { C_ENUM (LSM_MATHML_FONT_CMR10), "LSM_MATHML_FONT_CMR10", "cmr10" },
        { C_ENUM (LSM_MATHML_FONT_CMMI10), "LSM_MATHML_FONT_CMMI10", "cmmi10" },
        { C_ENUM (LSM_MATHML_FONT_CMEX10), "LSM_MATHML_FONT_CMEX10", "cmex10" },
        { C_ENUM (LSM_MATHML_FONT_CMSY10), "LSM_MATHML_FONT_CMSY10", "cmsy10" },
        { C_ENUM (LSM_MATHML_FONT_SYMBOL), "LSM_MATHML_FONT_SYMBOL", "symbol" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("LsmMathmlFont"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* Generated data ends here */

