
/* This file is generated by glib-mkenums, do not modify it. This code is licensed under the same license as the containing project. Note that it links to GLib, so must comply with the LGPL linking clauses. */

#include "lsmdomenumtypes.h"
#include "lsm.h"
#include "lsmtypes.h"
#include "lsmcairo.h"
#include "lsmstr.h"
#include "lsmutils.h"
#include "lsmmisc.h"
#include "lsmdebug.h"
#include "lsmtraits.h"
#include "lsmproperties.h"
#include "lsmattributes.h"
#include "lsmitex.h"
#include "lsmdomentities.h"
#include "lsmdom.h"
#include "lsmdomtypes.h"
#include "lsmdomnode.h"
#include "lsmdomnodelist.h"
#include "lsmdomnamednodemap.h"
#include "lsmdomdocument.h"
#include "lsmdomdocumentfragment.h"
#include "lsmdomelement.h"
#include "lsmdomcharacterdata.h"
#include "lsmdomtext.h"
#include "lsmdomview.h"
#include "lsmdomparser.h"
#include "lsmdomimplementation.h"

#define C_ENUM(v) ((gint) v)
#define C_FLAGS(v) ((guint) v)

/* enumerations from "lsmdebug.h" */

GType
lsm_debug_level_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (LSM_DEBUG_LEVEL_NONE), "LSM_DEBUG_LEVEL_NONE", "none" },
        { C_ENUM (LSM_DEBUG_LEVEL_WARNING), "LSM_DEBUG_LEVEL_WARNING", "warning" },
        { C_ENUM (LSM_DEBUG_LEVEL_DEBUG), "LSM_DEBUG_LEVEL_DEBUG", "debug" },
        { C_ENUM (LSM_DEBUG_LEVEL_LOG), "LSM_DEBUG_LEVEL_LOG", "log" },
        { C_ENUM (LSM_DEBUG_LEVEL_COUNT), "LSM_DEBUG_LEVEL_COUNT", "count" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("LsmDebugLevel"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* enumerations from "lsmdomnode.h" */

GType
lsm_dom_node_type_get_type (void)
{
    static gsize gtype_id = 0;
    static const GEnumValue values[] = {
        { C_ENUM (LSM_DOM_NODE_TYPE_ELEMENT_NODE), "LSM_DOM_NODE_TYPE_ELEMENT_NODE", "element-node" },
        { C_ENUM (LSM_DOM_NODE_TYPE_ATTRIBUTE_NODE), "LSM_DOM_NODE_TYPE_ATTRIBUTE_NODE", "attribute-node" },
        { C_ENUM (LSM_DOM_NODE_TYPE_TEXT_NODE), "LSM_DOM_NODE_TYPE_TEXT_NODE", "text-node" },
        { C_ENUM (LSM_DOM_NODE_TYPE_CDATA_SECTION_NODE), "LSM_DOM_NODE_TYPE_CDATA_SECTION_NODE", "cdata-section-node" },
        { C_ENUM (LSM_DOM_NODE_TYPE_ENTITY_REFERENCE_NODE), "LSM_DOM_NODE_TYPE_ENTITY_REFERENCE_NODE", "entity-reference-node" },
        { C_ENUM (LSM_DOM_NODE_TYPE_ENTITY_NODE), "LSM_DOM_NODE_TYPE_ENTITY_NODE", "entity-node" },
        { C_ENUM (LSM_DOM_NODE_TYPE_PROCESSING_INSTRUCTION_NODE), "LSM_DOM_NODE_TYPE_PROCESSING_INSTRUCTION_NODE", "processing-instruction-node" },
        { C_ENUM (LSM_DOM_NODE_TYPE_COMMENT_NODE), "LSM_DOM_NODE_TYPE_COMMENT_NODE", "comment-node" },
        { C_ENUM (LSM_DOM_NODE_TYPE_DOCUMENT_NODE), "LSM_DOM_NODE_TYPE_DOCUMENT_NODE", "document-node" },
        { C_ENUM (LSM_DOM_NODE_TYPE_DOCUMENT_TYPE_NODE), "LSM_DOM_NODE_TYPE_DOCUMENT_TYPE_NODE", "document-type-node" },
        { C_ENUM (LSM_DOM_NODE_TYPE_DOCUMENT_FRAGMENT_NODE), "LSM_DOM_NODE_TYPE_DOCUMENT_FRAGMENT_NODE", "document-fragment-node" },
        { C_ENUM (LSM_DOM_NODE_TYPE_NOTATION_NODE), "LSM_DOM_NODE_TYPE_NOTATION_NODE", "notation-node" },
        { 0, NULL, NULL }
    };
    if (g_once_init_enter (&gtype_id)) {
        GType new_type = g_enum_register_static (g_intern_static_string ("LsmDomNodeType"), values);
        g_once_init_leave (&gtype_id, new_type);
    }
    return (GType) gtype_id;
}

/* Generated data ends here */

